/* ivc 2010-12-21
 */

// Wire Slave Receiver
// by Nicholas Zambetti <http://www.zambetti.com>

// Receives data as an I2C/TWI slave device
// Refer to the "Wire Master Writer" example for use with this

// I2C / TWI Arduino Diecimila Pinout:
//
//    * Arduino analog input 4 = I2C SDA
//    * Arduino analog input 5 = I2C SCL 
//

#include <Wire.h>
#include "HL1606stripPWM.h"

int latchPin = 10;
HL1606stripPWM strip = HL1606stripPWM(128, latchPin);  // leds per meter, one meter 32 leds, latch pin
float brightness = 0.7;        // brightness, 0.0-1.0, lowest to brightest


String readString; //string for fetching data from address
char readArray[100];

void setup()
{
  Wire.begin(4);                // join i2c bus with address #4
  Wire.onReceive(receiveEvent); // register event
  //Serial.begin(9600);           // start serial for output

  // SI high on LED strip
  digitalWrite(8, 1);

    // set up strip configuration
  strip.setPWMbits(3);          // 4 bits is 16 PWM per LED, 12 bit color with
  // 4096 different colors available
  strip.setSPIdivider(32);      // spi update speed
  strip.setCPUmax(90);          // cpu utilitization for led driving
  strip.begin();
}

uint8_t j=0;
uint8_t meter=50;
uint8_t up, dn;
uint8_t oldval_up=1,oldval_dn=1;


void loop()
{
  driveLedStrip(dn, up);
  delay(100);
}

// function that executes whenever data is received from master, registered as an event
void receiveEvent(int howMany)
{

  char i=0;
  char data[20];

  String up_s, dn_s;
  //char *up_c, *dn_c;
  char up_c[100];
  char dn_c[100];
  String bwdata;

  while(Wire.available()) { // loop through all but the las
    char c = Wire.receive(); // receive byte as a character
    //store characters to string
    //readString.concat(c);
    data[i] = c;

    //Serial.print("char: ");
    //Serial.println(c);  
    i++;
  }

  //int x = Wire.receive();    // receive byte as an integer
  //Serial.print("integer: ");         // print the integer
  //Serial.println(x);         // print the integer
  //readString.concat(x);
  //data[i] = x;

  // end of line
  data[i] = 0;

  // print string
  Serial.print("Bandwidth data (up-down): ");
  //Serial.println(readString);
  Serial.println(data);

  // drive led strip
  //driveLedStrip(readString);
  

  bwdata = String(data);

  // get bandwidth values
  up_s = bwdata.substring(0,bwdata.lastIndexOf("#")+1);
  up_s.toCharArray(up_c,up_s.length());
  dn_s = bwdata.substring(bwdata.lastIndexOf("#")+1);
  dn_s.toCharArray(dn_c,dn_s.length()+1);

  /*
  // find split character
   boolean found = false;
   char i;
   for (i=0;i<sizeof(bwdata)-1;i++) {
   if (&bwdata[i] == "#") found = true;
   if (!found) up_c[i] = bwdata[i];
   else dn_c[(i-sizeof(up_c))] = bwdata[i+1];
   Serial.print(i);
   }
   //up_c[sizeof(up_c)] = 0;
   //dn_c[sizeof(dn_c)] = 0;
   */


  // get int from char
  up = atoi(up_c);
  dn = atoi(dn_c);

  // print up and down data
  Serial.print("Up int: ");
  Serial.println(up, DEC);
  Serial.print("Down int: ");
  Serial.println(dn, DEC);
}


void driveLedStrip(uint8_t dn,uint8_t up) {

  //dn = dn*10;
  if (dn > 100) dn = 100;
  if (up > 100) up = 100;
  createMeter(up, 0, 255, 0, 0); // upload, second part of strip
  createMeter(dn, 0, 255, 0, 64); // download, start of strip
  
}

void createMeter(int8_t n, uint8_t r, uint8_t g, uint8_t b, uint8_t offset) {

  uint8_t i, k;
  uint8_t traildelay = 0;

  // offset split strip
  uint8_t leds = strip.numLEDs()/2; // 128/2 = 64 leds per bwmeter
  
  // find the number of active leds to be lit, n is the percentage passed to function
  uint8_t ledsactive = (n/100.0)*leds; // value range 0-64

  // set brightness
  float bright = brightness/100.0; // 0-100%

  // download meter
  // reverse led display order for "dn", led 127-64 (64 total)
  if (offset > 0) {
    // set led pixel
    if (ledsactive > oldval_dn) traildelay = 320/(ledsactive-oldval_dn);
    else traildelay = 320/(oldval_dn-ledsactive);

    k = 0;
    for (i=oldval_dn; i<ledsactive; i++) {
      if (i < leds*0.7) strip.setLEDcolorPWM(offset+63-i, 0, 255*bright, 0); // stage 1 green
      else if (i < leds*0.9) strip.setLEDcolorPWM(offset+63-i, 255*bright, 165*bright, 0); // stage 2 amber
      else strip.setLEDcolorPWM(offset+63-i, 255*bright, 0, 0); // stage 3 red

      // delay decreases with more leds, faster swips
      if (traildelay-k > 0) delay(traildelay-k);
      if (traildelay > 5) k += traildelay*0.1;
    }

    // clear strip, reset the other unused leds
    k = 0;
    for (i=oldval_dn; i>ledsactive; i--) {
      strip.setLEDcolorPWM(offset+63-i+1, 0, 0, 0);
      // delay
      if (traildelay-k > 0) delay(traildelay-k);
      if (traildelay > 5) k += traildelay*0.1;
    }
    // update old value variable for next loop update
    oldval_dn = ledsactive;
    
  } else {
    // upload meter
    // not reverse for "up" meter, led 0-63 (64 total)
    if (ledsactive > oldval_up) traildelay = 320/(ledsactive-oldval_up);
    else traildelay = 320/(oldval_up-ledsactive);
    
    k = 0;
    for (i=oldval_up; i<ledsactive; i++) {
      if (i < leds*0.7) strip.setLEDcolorPWM(i, 0, 255*bright, 0); // stage 1 green
      else if (i < leds*0.9) strip.setLEDcolorPWM(i, 255*bright, 165*bright, 0); // stage 2 amber
      else strip.setLEDcolorPWM(i, 255*bright, 0, 0); // stage 3 red
      // delay decreases with more leds, faster swips
      if (traildelay-k > 0) delay(traildelay-k);
      if (traildelay > 5) k += traildelay*0.1;
    }

    // clear strip, reset the other unused leds
    k = 0;
    for (i=oldval_up; i>ledsactive; i--) {
      strip.setLEDcolorPWM(i-1, 0, 0, 0);
      // delay
      if (traildelay-k > 0) delay(traildelay-k);
      if (traildelay > 5) k += traildelay*0.1;
    }
    // update old value variable for next loop update
    oldval_up = ledsactive;
  }
  

 // gammel verdi 96
 // ny verdi 43
 // diff ned 53

}




